from .visual_testing_agent import start_visual_testing_agent
from .functional_testing_agent import start_functional_testing_agent

def start_testing():
    start_visual_testing_agent()
    start_functional_testing_agent()

